package abstractions;

import tools.HasInvariant;

/**
 * Provides all the common code for implementing a {@link BalanceSpecification} but abstracts
 * away from the weigh mechanism {@link BalanceSpecification#weigh} that tries to balance a target weight
 * @author gibson
 * @version 1
 */
public abstract class Balance implements BalanceSpecification, HasInvariant{
	
	private int [] left;
	private int [] right;
	private int [] table;
	
	final protected int MAX_NUMBER_OF_WEIGHTS;
    protected int number_of_weights;
	protected int maxThatCanBeWeighed;
	
	
	public boolean invariant (){
		
		if (MAX_NUMBER_OF_WEIGHTS<1) return false;
		if (MAX_NUMBER_OF_WEIGHTS<number_of_weights) return false;
		if (maxThatCanBeWeighed < MAX_NUMBER_OF_WEIGHTS) return false;
		if (left.length != right.length) return false;
		if (left.length != table.length) return false;
		
		for (int i =0; i <number_of_weights; i++){
			if (left[i] <0 || right[i] <0 || table[i] <0) return false;
			if (left[i] >0 && right[i] >0) return false;
			if (left[i] >0 && table[i] >0) return false;
			if (table[i] >0 && right[i] >0) return false;
			if (left[i] ==0 && right[i] == 0 && table[i] ==0) return false;
		}
		
		return true;
	}
	
	/**
	 * Initially, there are no weights in the system (in left or right pans or on table)
	 * @param maxNumberOfWeights is the maximum number of weights that can be on the table 
	 * or in the pans
	 * @throws IllegalArgumentException if the maxNumberOfWeights is not a positive integer;
	 */
	public Balance (int maxNumberOfWeights) throws IllegalArgumentException{
		if (maxNumberOfWeights <=0) throw new IllegalArgumentException("There must be at least one weight in the system");
		this.MAX_NUMBER_OF_WEIGHTS = maxNumberOfWeights;
		left = new int [maxNumberOfWeights];
		right = new int [maxNumberOfWeights];
		table = new int [maxNumberOfWeights];
		maxThatCanBeWeighed =0;
	}
	
	/**
	 * 
	 * @param weights is the new weight set to be added to the table after all the other weights are removed
	 * @throws IllegalArgumentException if:
	 * <ul> <li> the weight set is bigger than the number of weights allowed
	 *      <li> the weight set would cause an integer overflow if the total of all weights is calculated
	 *      <li> the weight set contains a negative value
	 * </ul>
	 */
	protected  void initialiseWeights ( int  [] weights) throws IllegalArgumentException{
		
		if (MAX_NUMBER_OF_WEIGHTS < weights.length) 
			throw new IllegalArgumentException("There are too many weights for the system");
		
		number_of_weights = weights.length;
		maxThatCanBeWeighed =0;
		
		for (int i=0; i<number_of_weights; i++){
			left[i] =0;
		    right[i]=0;
		    table [i] = weights[i];
		    if (weights[i]<0) throw new IllegalArgumentException("Negative weights are not permitted");
		    maxThatCanBeWeighed = maxThatCanBeWeighed+weights[i];
		    
		    if (maxThatCanBeWeighed <0) 
		    	throw new IllegalArgumentException("The overall weight is too big and risks causing an integer overflow");
		    
		    }
		
	}

	
	@Override
	public void randomize(){
		
    	moveAllToTable();
    	
    	double  randomRoll;
    	for (int i=0;i<maxNumberOfWeights();i++){
    		
    		randomRoll = Math.random();
    		
    		if (randomRoll<0.333) {left[i] = table[i]; table[i] =0;}
    		if (randomRoll<0.666) {right[i] = table[i]; table[i] =0;}
    	}
    		
	}
	
	
	@Override
	public void removeLeft (int x){
		
		moveFromTo(left, table, x);
	}
	
	@Override
	public void removeRight (int x){

		moveFromTo(right, table, x);
	}
	
	@Override
	public void addLeft (int x){

		moveFromTo(table, left, x);
	}

	@Override
	public void addRight (int x){

		moveFromTo(table, right, x);
	}
	
	@Override
	public int weighing(){
		
		return totalWeightValuesInSameLocation(left) - totalWeightValuesInSameLocation(right);
	}
	
	@Override
	public int maxNumberOfWeights() {
		
		return MAX_NUMBER_OF_WEIGHTS;
	}

	@Override
	public int maxThatCanBeWeighed() {
	
		return maxThatCanBeWeighed;
	}
	

	@Override
	public int numberOfWeights() {
		
		return number_of_weights;
	}


	public String toString(){
		
		String str="\n";
		str = str+ "Left : "+ displayWeights(left);
		str = str+ "\nRight : "+ displayWeights(right);
		str = str+ "\nTable : "+ displayWeights(table);
		return str;
	}
	
	@Override
	public void moveAllToTable() {
		

		for (int i=0; i<number_of_weights; i++){
			
			if (left[i]>0) removeLeft(left[i]);
			if (right[i]>0) removeRight(right[i]);
		}

	}
	
	@Override
	public int getWeight(int index) throws IllegalArgumentException{
		
		if (index<0 || index> number_of_weights) 
			throw (new IllegalArgumentException("index is not valid"));
		
		return (left[index]+right[index]+table[index]);
	}
	

		
	/**
	 * @param array specifies the location (either the left pan the right pan or the table)
	 * @return the total of all the weights in the location array specified
	 */
	private int totalWeightValuesInSameLocation(int [] array){
	
		int total =0;	
		for (int i =0; i <number_of_weights; i++) total = total+ array[i];
	    return total;		  
	}
	
	/**
	 * @param array is to be searched 
	 * @param x is the value we are searching for
	 * @return an index for the location in the array where the value is found, or -1 if not found
	 * @throws IllegalArgumentException if we try to find a negative weight
	 */
	private int findWeight (int [] array, int x) throws IllegalArgumentException{
      if (x< 0) throw (new IllegalArgumentException("no negative weights in system"));
      for (int i =0; i <number_of_weights; i++)
    	  if ( x == array[i] ) return i;
	  return -1;  
	}
	
	/**
	 * @param array  identifies the location of the weights to be displayed
	 * @return the weights in a space separated list, 
	 */
	private String displayWeights (int [] array){
		
		String display = "";
		for (int i =0; i <number_of_weights; i++)  if (array[i] !=0) display = display + " "+array[i];
		return display;
	}
	
	/**
	 * @param array specifies the location (either the left pan the right pan or the table)
	 * @param x is the weight to be added to the specified location 
	 * @param index is the position in the array where it should be added
	 */
	private void addWeightToLocation (int [] array, int x, int index){
		
	     array[index] = x; 
	}
	

	/**
	 * @param from specifies where we look for the weight to move
	 * @param to specifies where we should move the weight (if we find it)
	 * @param x is the weight we wish to move
	 */
	private void moveFromTo (int [] from , int [] to , int x){
		
       if (from == to) return; //nothing to do
		
	   int indexOfWeight = findWeight(from, x);
	   	
	   if (indexOfWeight >=0){
		addWeightToLocation(to, from[indexOfWeight], indexOfWeight );
		from[indexOfWeight] =0;
	    }
	}
	
}
