package abstractions;

import tools.HasInvariant;

/**
 * A balance has 2 pans - left and right -  and a set of weights which can be in 1 of the 2 pans or unused (on the table)
 * on the table. The weight set cannot be changed after initialisation and weights can be moved between the table and the pans.
 * Initially all weights should be on the table.
 * 
 * @author J Paul Gibson
 * @version 1
 */
public interface BalanceSpecification extends HasInvariant{
	

	/**
	 * Search for the specified weight in the left pan.
	 * If the weight is found then move it back to the table.
	 * Otherwise do nothing.
	 * @param x is the weight that we wish to move
	 */
	public void removeLeft (int x);
	
	/**
	 * Search for the specified weight in the right pan.
	 * If the weight is found then move it back to the table.
	 * Otherwise do nothing.
	 * @param x is the weight that we wish to move
	 */
	
	public void removeRight (int x);
	
	/**
	 * Search for the specified weight on the table.
	 * If the weight is found then move it to the left pan.
	 * Otherwise do nothing.
	 * @param x is the weight that we wish to move
	 */
	public void addLeft (int x);
	
	/**
	 * Search for the specified weight on the table.
	 * If the weight is found then move it to the right pan.
	 * Otherwise do nothing.
	 * @param x is the weight that we wish to move
	 */
	public void addRight (int x);
	
	/**
	 * This method should move the weights around in order to weigh this amount. 
	 * If no exact solution is found then it should find a solution as close as possible to the one required.
	 * @param x is the weight that we wish to weigh in the right pan. 
	 * @throws IllegalArgumentException if the target weight is negative 
	 */
	public void weigh(int x) throws IllegalArgumentException;
	
	/**
	 * @return the difference between the total weights in the left and right pans
	 */
	public int weighing();
	
	/**
	 * Move all weights in left and right pans to the table
	 */
	public void moveAllToTable();
	
	/**
	 * @return the maximum number of weights shared between the table and the pans -
	 * this value should be constant after the system is initialised
	 */
	public int maxNumberOfWeights();
	
	/**
	 * @return the actual number of weights shared between the table and the pans 
	 */
	public int numberOfWeights();
	
	/**
	 * @return the maximum value that can be weighed : the total of all the weights
	 */
	public int maxThatCanBeWeighed();
	
	/**
	 * @param i is the index for the weight 
	 * @return the value of the ith weight in the balance system
	 * @throws IllegalArgumentException if the index is not in range 0 .. maxNumberOfWeights-1
	 */
	public int getWeight(int i) throws IllegalArgumentException;
	
	/**
	 * Place the current weights randomly with each weight having an equal chance of going in each of
	 * the three pans (left, right or table)
	 */
	public void randomize();
	
	/**
	 * 
	 * @return the state of the balance following the format (as for a 10 weight boolean system
	 * with weights shared between the 3 possible locations):
	   <pre> 
		Left : 1 8
		Right : 128 256 512
		Table : 2 4 16 32 64 1024 
	    </pre>
	    */
	public String toString ();
	
}
