package models;

import abstractions.Balance;
import abstractions.BalanceSpecification;

/**
 * Implements a balance {@link BalanceSpecification} where the set of weights 
 * will have binary integer values from <code> 1,2,4 .. 2^(number-1) </code><br>
 * The balancing can be done by placing weights in only side of the balance  
 * (with the target in the other side)
 * @author gibson
 * @version 1
 */
public class BinaryBalance extends Balance{
	
	/**
	 * As ints are 32 bit, the total weight of all binary weights will cause an overflow if we have more than 31 weights
	 */
	public final static int MAXIMUM_NUMBER_OF_BINARY_WEIGHTS = 31;
	
	
	/**
	 * Strengthen the invariant to ensure that the weights are binary, do not exceed the maximum size
	 *  and are in the correct order:
	 * <code> 1,2,4 .. 2^(number-1) </code>
	 */
	public boolean invariant(){
		
		if (!super.invariant()) return false;
		
		if (number_of_weights > MAXIMUM_NUMBER_OF_BINARY_WEIGHTS) return false;
		
		int powerOf2 = 1;
		for (int i = 0; i< number_of_weights; i++){
			if (getWeight(i)!= powerOf2) return false;
			powerOf2 = powerOf2 *2;
		}
			
		return true;
	}
	

  /**
   * @param number is the number of weights to be used in the weighing pans.
   * All these weights will have binary integer values from <code> 1 ... 2^(number-1) </code>
   * @throws IllegalArgumentException if:
   * <ul>
   * <li> the number is not a positive integer,
   * <li> the number is too large as it would result in an integer overflow error
   * </ul>
   */
	public BinaryBalance(int number) throws IllegalArgumentException {
		
		super(number);  // throws exception of the number is not positive
		
		int [] weights = new int [number];
		int power2 = 1;
		                          
		for (int i = 0; i< number; i++){
			
			weights[i] = power2; power2=power2*2;

		}
		
		initialiseWeights(weights);// throws exception if sum of all weights would result in overflow

	}

    /**
     * In a binary system we can weigh the required amount by placing weights in the left pan only<br>
     * If the target weight is bigger than the total weight then add all weights to the left.
     */
	@Override
	public void weigh(int x) throws IllegalArgumentException{
		
		if (x<0) throw new IllegalArgumentException("target weight must be a non-negative integer");
		
		moveAllToTable();
		boolean needAll = (x >= maxThatCanBeWeighed);
			
		int power2=1;
		while (x>0){
			
			if (x%2 ==1 || needAll) addLeft(power2);
			power2 = power2*2;
			x = x/2;
		}	
	}



}

