package models;

import abstractions.Balance;
import abstractions.BalanceSpecification;

/**
 * Implements a balance {@link BalanceSpecification} where the set of weights 
 * will have integer values from <code>1 .. max</code><br>
 * The balancing can be done by placing weights in only side of the balance  
 * (with the target in the other side)
 * @author gibson
 * @version 1
 */
public class SequenceBalance extends Balance{

	/**
	 * As ints are 32 bit, the total weight of all binary weights will cause an overflow if we have more than 65535 weights
	 */
	public final static int MAXIMUM_NUMBER_OF_SEQUENCE_WEIGHTS =   65535;
	
	
	/**
	 * Strengthen the invariant to ensure that the weights are in sequence and do not exceed the maximum size:
	 * <code> 1,2,3 .. number </code>
	 */
	public boolean invariant(){
		
		if (!super.invariant()) return false;
		
		if (number_of_weights > MAXIMUM_NUMBER_OF_SEQUENCE_WEIGHTS) return false;
		
		for (int i = 0; i< number_of_weights; i++){
			if (getWeight(i)!= i+1) return false;
		}
			
		return true;
	}
	
	  /**
	   * @param number is the number of weights to be used in the weighing pans.
	   * All these weights will have binary integer values from <code> 1 ... 2^(number-1) </code>
	   * @throws IllegalArgumentException if:
	   * <ul>
	   * <li> the number is not a positive integer,
	   * <li> the number is too large as it would result in an integer overflow error
	   * </ul>
	   */
		public SequenceBalance(int number) throws IllegalArgumentException {
			
			super(number);  // throws exception of the number is not positive
		
		    int [] weights = new int [number];
		                       
		    for (int i = 0; i< number; i++)
			weights[i] = i+1;

		    initialiseWeights(weights);// throws exception if sum of all weights would result in overflow
	     }

		/**
	     * In a binary system we can weigh the required amount by placing weights in the left pan only<br>
	     * If the target weight is bigger than the total weight then add all weights to the left.
	     */
		@Override
		public void weigh(int x) throws IllegalArgumentException{
			
			if (x<0) throw new IllegalArgumentException("target weight must be a non-negative integer");
			
			moveAllToTable();
	
			if (x > maxThatCanBeWeighed)
			    for (int i = 0; i < number_of_weights;i++) addLeft(i+1);
		
	            else{ int target = x;
		              int biggest = number_of_weights;
		      
		               while (target>0){  if (target >= biggest) {
		    	                                                   addLeft(biggest);
		    	                                                   target = target - biggest;
		    	                                                 }
		                                   biggest --; 
		                               }
		           }  
	     }
}
