package models;

import abstractions.Balance;
import abstractions.BalanceSpecification;

/**
 * Implements a balance {@link BalanceSpecification} where the set of weights 
 * will have ternary integer values from <code> 1 ... 3^(number-1) </code><br>
 * The balancing can be done by placing weights in both sides of the balance  
 * @author gibson
 * @version 1
 */
public class TernaryBalance extends Balance{

	/**
	 * As ints are 32 bit, the total weight of all ternary weights will cause an overflow if we have more 20 weights
	 */
	public final static int MAXIMUM_NUMBER_OF_TERNARY_WEIGHTS = 20;
	
	  /**
	   * @param number is the number of weights to be used in the weighing pans.
	   * All these weights will have binary integer values from <code> 1 ... 3^(number-1) </code>
	   * @throws IllegalArgumentException if:
	   * <ul>
	   * <li> the number is not a positive integer,
	   * <li> the number is too large as it would result in an integer overflow error
	   * </ul>
	   */
	public TernaryBalance(int number) throws IllegalArgumentException{
		
		super(number);  // throws exception if the number is not positive
		
		int [] weights = new int [number];
		int power3 = 1;
		                          
		for (int i = 0; i< number; i++){
			
			weights[i] = power3;
			power3=power3*3;
		}
		
		initialiseWeights(weights); // throws exception if sum of all weights would result in overflow

	}
	
	/**
	 * Strengthen the invariant to ensure that the weights are ternary, do not exceed the maximum size
	 *  and are in the correct order:
	 * <code> 1,3,9 .. 3^(number-1) </code>
	 */
	public boolean invariant(){
		
		if (!super.invariant()) return false;
		
		if (number_of_weights > MAXIMUM_NUMBER_OF_TERNARY_WEIGHTS) return false;
		
		int powerOf3 = 1;
		for (int i = 0; i< number_of_weights; i++){
			if (getWeight(i)!= powerOf3) return false;
			powerOf3 = powerOf3 *3;
		}
			
		return true;
	}
	
    /**
     * In a ternary system we can weigh the required amount by placing weights in the left and right pans<br>
     * If the target weight is bigger than the total weight then add all weights to the left.
     */
	@Override
	public void weigh(int x) throws IllegalArgumentException{
		
		if (x<0) throw new IllegalArgumentException("target weight must be a non-negative integer");
		
		moveAllToTable();
		
		if (x > maxThatCanBeWeighed){
			int power3 =1;
			for (int i = 0; i < number_of_weights+1;i++)
			    {addLeft(power3);power3=power3*3;}
		}
		
		else split(x, 'L');	
	}

	
	private static char flip (char side){
		
		if (side == 'L') return 'R';
		                 else return 'L';
	}
	
	
	private void split (int target, char side){
		
		if (target ==0) return;
		
		int power3 =1;
		int previouspower3=0;
		
		while (power3 < target && power3>0){previouspower3=power3; power3=power3*3;}
		
		// Overflow in calculating power3 so we have to revert to previous value
		if (power3<1){
		
		    addLeft(previouspower3);
		    split(target-previouspower3, side); 
			return;	}
		
		if (target  == power3){
			if (side =='L') addLeft(target); else addRight(target);	
			return;}
		
		if (target <= power3/2 ){
			if (side =='L') addLeft(previouspower3); else addRight(previouspower3);	
			split(target-previouspower3, side); 
			return;}
		
		else {if (side =='L') addLeft(power3); else addRight(power3); 
              split(power3-target, flip(side)); 
              return;}
		
	}
}
