package tests;

import tools.DateHeader;
import models.BinaryBalance;

/**
 * Validation test for {@link BinaryBalance}<br>
 * The test is run for a default target value of 100. <br>
 * This is to represent the balance in the state:
 <pre>
   \4,32,64/            \100/
    -------             -----
       |                  |
       --------------------
                /\
 </pre>
 * (This value can be over-written by a valid integer parameter on the command line.) <br>
 * For the default value the expected (time-stamped) output is:
 * <pre>
The target weight is set, by default, to 100.
You can override this default value by passing an integer value as a main argument parameter, if you so wish.

********************************************************************
Execution Date/Time 2014/02/21 17:11:12
*********************************************************************

 **** Testing the biggest possible weight set ****

Left : 
Right : 
Table :  1 2 4 8 16 32 64 128 256 512 1024 2048 4096 8192 16384 32768 65536 131072 262144 524288 1048576 2097152 4194304 8388608 16777216 33554432 67108864 134217728 268435456 536870912 1073741824
Weighing 0
balance.maxNumberOfWeights() =  31
balance.maxThatCanBeWeighed() =  2147483647

 **** Testing weighing of 100  ****

Left :  4 32 64
Right : 
Table :  1 2 8 16 128 256 512 1024 2048 4096 8192 16384 32768 65536 131072 262144 524288 1048576 2097152 4194304 8388608 16777216 33554432 67108864 134217728 268435456 536870912 1073741824

 **** Testing weighing of maximum that can be weighed ****

Left :  1 2 4 8 16 32 64 128 256 512 1024 2048 4096 8192 16384 32768 65536 131072 262144 524288 1048576 2097152 4194304 8388608 16777216 33554432 67108864 134217728 268435456 536870912 1073741824
Right : 
Table : 
Weighing 2147483647

 **** Testing randomisation of balance ****

Left :  1 4 8 32 512 1024 524288 1048576 2097152 67108864 268435456
Right :  2 128 2048 8192 16384 262144 8388608 16777216
Table :  16 64 256 4096 32768 65536 131072 4194304 33554432 134217728 536870912 1073741824
Weighing 313761195

 **** Testing the smallest possible weight set ****

Left : 
Right : 
Table :  1
Weighing 0
balance.maxNumberOfWeights() =  1
balance.maxThatCanBeWeighed() =  1

 **** Testing weighing of 1  ****

Left :  1
Right : 
Table : 

 **** Testing weighing of 100  ****

Left :  1
Right : 
Table : 

 **** Testing exceptions in balance construction ****

 Correctly caught the overflow case: 
java.lang.IllegalArgumentException: Negative weights are not permitted

 Correctly caught the negative weights exception: 
java.lang.IllegalArgumentException: There must be at least one weight in the system
</pre>
 * 
 * @author gibson
 * @version 1
 */
public class TestBinaryBalance {
	
	
public static void main (String[] args){
	
	// The default target sum to weigh on the balance
	final int DEFAULT_TARGET = 100;
	
    int target = DEFAULT_TARGET;
    		
    if (args.length ==0) {
		System.out.println("The target weight is set, by default, to "+target+
      			           ".\nYou can override this default value by passing an integer value as "
      			           + "a main argument parameter, if you so wish.");  	
    }
    
    else try{target = Integer.parseInt(args[0]);
             if (target<1){
            	 System.out.println("The target weight must be greater than 0, "
            	 		+ "so we default the value to "+ DEFAULT_TARGET+".\n");
            	 target = DEFAULT_TARGET;}
             else System.out.println("The target weight has been set to "+target);
    }
         catch (NumberFormatException exc){
        	   System.out.println("First argument was not a valid integer");
               System.out.println("The target weight is specifed by default to "+target);
         }
         
    
	System.out.println(DateHeader.dateString());    


	    
		BinaryBalance balance = new BinaryBalance (BinaryBalance.MAXIMUM_NUMBER_OF_BINARY_WEIGHTS);
		
		System.out.println("\n **** Testing the biggest possible weight set ****");
		System.out.println(balance);
		System.out.println("Weighing "+balance.weighing());
		System.out.println("balance.maxNumberOfWeights() =  "+balance.maxNumberOfWeights());
		System.out.println("balance.maxThatCanBeWeighed() =  "+balance.maxThatCanBeWeighed());
		
		// We can weigh any value by just putting weights in left balance, when target is in right
		
		balance.weigh(target);
		System.out.println("\n **** Testing weighing of "+ target+"  ****");
		System.out.println(balance);
		
		System.out.println("\n **** Testing weighing of maximum that can be weighed ****");
	    balance.weigh(balance.maxThatCanBeWeighed());
	    System.out.println(balance);
	    System.out.println("Weighing "+balance.weighing());
		
		System.out.println("\n **** Testing randomisation of balance ****");
		balance.randomize();System.out.println(balance);
		System.out.println("Weighing "+balance.weighing());
		
		System.out.println("\n **** Testing the smallest possible weight set ****");
		
		/**
	     * the size of the smallest binary weight set allowed
	     */
		final int MIN_NUMBER_OF_WEIGHTS = 1;
		
        balance = new BinaryBalance (MIN_NUMBER_OF_WEIGHTS);
        
		System.out.println(balance);
		System.out.println("Weighing "+balance.weighing());
		System.out.println("balance.maxNumberOfWeights() =  "+balance.maxNumberOfWeights());
		System.out.println("balance.maxThatCanBeWeighed() =  "+balance.maxThatCanBeWeighed());
		
		target = balance.maxThatCanBeWeighed();
		balance.weigh(target);
		System.out.println("\n **** Testing weighing of "+ target+"  ****");
		System.out.println(balance);

		target = DEFAULT_TARGET;
		balance.weigh(target);
		System.out.println("\n **** Testing weighing of "+ target+"  ****");
		System.out.println(balance);
		
		System.out.println("\n **** Testing exceptions in balance construction ****");
		
		 try {balance = new BinaryBalance (BinaryBalance.MAXIMUM_NUMBER_OF_BINARY_WEIGHTS+1);}
		 catch (IllegalArgumentException exception )
		       {System.out.println("\n Correctly caught the overflow case: \n" +exception);}
		 
	     try {balance = new BinaryBalance (-1);}
		 catch (IllegalArgumentException exception )
			 {System.out.println("\n Correctly caught the negative weights exception: \n"+exception);}
    }
}
