package tests;

import models.SequenceBalance;
import tools.DateHeader;

/**
 * Validation test for {@link SequenceBalance}<br>
 * The test is run for a default target value of 1000. <br>
 * This is to represent the balance in the state:
 <pre>
\128 127 126 125 124 123 122 121 4/              \1000/
 ---------------------------------                -----
       |                                            |
       ----------------------------------------------
                           /\
 </pre>
 * (This value can be over-written by a valid integer parameter on the command line.) <br>
 * For the default value the expected (time-stamped) output is:
 * <pre>
The target weight is set, by default, to 1000.
You can override this default value by passing an integer value as a main argument parameter, if you so wish.

********************************************************************
Execution Date/Time 2014/02/21 17:48:44
*********************************************************************

 **** Testing a reasonably size weight set (the biggest would fill the screen with values) ****

Left : 
Right : 
Table :  1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128
Weighing 0
balance.maxNumberOfWeights() =  128
balance.maxThatCanBeWeighed() =  8256

 **** Testing weighing of 1000  ****

Left :  4 121 122 123 124 125 126 127 128
Right : 
Table :  1 2 3 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120

 **** Testing the smallest possible weight set ****

Left : 
Right : 
Table :  1
Weighing 0
balance.maxNumberOfWeights() =  1
balance.maxThatCanBeWeighed() =  1

 **** Testing weighing of 1  ****

Left :  1
Right : 
Table : 

 **** Testing weighing of 1000  ****

Left :  1
Right : 
Table : 

 **** Testing exceptions in balance construction ****

 Correctly caught the overflow case: 
java.lang.IllegalArgumentException: The overall weight is too big and risks causing an integer overflow

 Correctly caught the negative weights exception: 
java.lang.IllegalArgumentException: There must be at least one weight in the system
</pre>
 * 
 * @author gibson
 * @version 1
 */
public class TestSequenceBalance {
	
	
public static void main (String[] args){
	
	// The default target sum to weigh with the balance
	final int DEFAULT_TARGET = 1000;
	
    int target = DEFAULT_TARGET;
    		
    if (args.length ==0) {
		System.out.println("The target weight is set, by default, to "+target+
      			           ".\nYou can override this default value by passing an integer value as "
      			           + "a main argument parameter, if you so wish.");  	
    }
    
    else try{target = Integer.parseInt(args[0]);
             if (target<1){
            	 System.out.println("The target weight must be greater than 0, "
            	 		+ "so we default the value to "+ DEFAULT_TARGET+".\n");
            	 target = DEFAULT_TARGET;}
             else System.out.println("The target weight has been set to "+target);
    }
         catch (NumberFormatException exc){
        	   System.out.println("First argument was not a valid integer");
               System.out.println("The target weight is specifed by default to "+target);
         }
         
    
	System.out.println(DateHeader.dateString());    

	    
		SequenceBalance balance = new SequenceBalance (128);
		
		System.out.println("\n **** Testing a reasonably size weight set (the biggest would fill the screen with values) ****");
		System.out.println(balance);
		System.out.println("Weighing "+balance.weighing());
		System.out.println("balance.maxNumberOfWeights() =  "+balance.maxNumberOfWeights());
		System.out.println("balance.maxThatCanBeWeighed() =  "+balance.maxThatCanBeWeighed());
		
		// We can weigh any value by just putting weights in left balance, when target is in right
		
		balance.weigh(target);
		System.out.println("\n **** Testing weighing of "+ target+"  ****");
		System.out.println(balance);

		
		System.out.println("\n **** Testing the smallest possible weight set ****");
		
		/**
	     * the size of the smallest binary weight set allowed
	     */
		final int MIN_NUMBER_OF_WEIGHTS = 1;
		
        balance = new SequenceBalance (MIN_NUMBER_OF_WEIGHTS);
        
		System.out.println(balance);
		System.out.println("Weighing "+balance.weighing());
		System.out.println("balance.maxNumberOfWeights() =  "+balance.maxNumberOfWeights());
		System.out.println("balance.maxThatCanBeWeighed() =  "+balance.maxThatCanBeWeighed());
		
		target = balance.maxThatCanBeWeighed();
		balance.weigh(target);
		System.out.println("\n **** Testing weighing of "+ target+"  ****");
		System.out.println(balance);

		target = DEFAULT_TARGET;
		balance.weigh(target);
		System.out.println("\n **** Testing weighing of "+ target+"  ****");
		System.out.println(balance);
		
		System.out.println("\n **** Testing exceptions in balance construction ****");
		
		 try {balance = new SequenceBalance (SequenceBalance.MAXIMUM_NUMBER_OF_SEQUENCE_WEIGHTS+1);}
		 catch (IllegalArgumentException exception )
		       {System.out.println("\n Correctly caught the overflow case: \n" +exception);}
		 
	     try {balance = new SequenceBalance (-1);}
		 catch (IllegalArgumentException exception )
			   {System.out.println("\n Correctly caught the negative weights exception: \n"+exception);}
    }
}
